import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';

import { Observable, of } from 'rxjs';
import { catchError, map, tap } from 'rxjs/operators';
import { Student } from './student';
import { Grade } from './grade'; 

@Injectable({
  providedIn: 'root'
})
export class GenericService {
  private backendUrl = 'http://localhost/wp/pdo-mvc/controller/controller.php';  // URL to web api 
  httpOptions = {
   	  headers: new HttpHeaders({ 
   	  		'Content-Type': 'application/json'
   	  })
  }; 

  constructor(private http: HttpClient) { }

  fetchStudents() : Observable<Student[]> {
  	  /* body of the method */
  	  return this.http.get<Student[]>(this.backendUrl+'?action=getAllStudents')
  	  		.pipe(catchError(this.handleError<Student[]>('fetchStudents', []))
      );
  } 

  fetchStudentsWithGrades() : Observable<Student[]> {
      /* body of the method */
      return this.http.get<Student[]>(this.backendUrl+'?action=getAllGrades')
          .pipe(catchError(this.handleError<Student[]>('fetchStudentsWithGrades', []))
      );
  }  


 UpdateGrade(stud: Student, grade: Grade): Observable<string> {
      return this.http.get<string>(this.backendUrl+'?action=updateGradeForStudent&studentid='+
          stud.id+'&course='+grade.course+'&grade='+grade.grade)
          .pipe(catchError(this.handleError<string>('UpdateGrade', ""))
      );
  }

  DeleteGrade(stud: Student, grade: Grade): Observable<string> {
      return this.http.get<string>(this.backendUrl+'?action=removeGradeForStudent&studentid='+
          stud.id+'&course='+grade.course)
          .pipe(catchError(this.handleError<string>('DeleteGrade', ""))
      );
  }

  AddGrade(stud: Student, grade: Grade): Observable<string> {
      return this.http.get<string>(this.backendUrl+'?action=addGradeForStudent&studentid='+
          stud.id+'&course='+grade.course+'&grade='+grade.grade)
          .pipe(catchError(this.handleError<string>('AddGrade', ""))
      );
  }
 


  /**
  * Handle Http operation that failed.
  * Let the app continue.
  * @param operation - name of the operation that failed
  * @param result - optional value to return as the observable result
  */
  private handleError<T>(operation = 'operation', result?: T) {
	  	return (error: any): Observable<T> => {

	    // TODO: send the error to remote logging infrastructure
	    console.error(error); // log to console instead

    	// Let the app keep running by returning an empty result.
    	return of(result as T);
    };
  } 
}
