import { Component, OnInit } from '@angular/core';
import { Student } from '../student';
import { Grade } from '../grade';
import { GenericService } from '../generic.service';
 
@Component({
  selector: 'app-students-grades',
  templateUrl: './students-grades.component.html',
  styleUrls: ['./students-grades.component.scss']
})
export class StudentsGradesComponent implements OnInit {
  students : Student[] = [];
  selectedStudent?: Student;
  //newgrade : Grade = {studentid: -1, course: "", grade: 0};
  errorMessage: string = "";

  constructor(private genericService: GenericService) { }

  ngOnInit(): void {
  	this.getStudents();
  }

  getStudents(): void {
  	this.genericService.fetchStudentsWithGrades()
    	.subscribe(students => this.students = students);
  }

  onSelect(stud: Student): void {
    this.selectedStudent = stud;
  }

  UpdateGrade(stud: Student, grade: Grade): void {
    this.errorMessage = "";
    this.genericService.UpdateGrade(stud, grade)
      .subscribe(response => {
          let r: any = response;
          this.errorMessage = r.result;
      })
  }

  DeleteGrade(stud: Student, grade:Grade): void {
    this.errorMessage = "";
    this.genericService.DeleteGrade(stud, grade)
      .subscribe(response => {
          let r: any = response;
          this.errorMessage = r.result;
      })
  }

  AddGrade(stud: Student, courseInputElem, gradeInputElem): void {
    this.errorMessage = "";
    let newgrade : Grade = {studentid: stud.id, course: courseInputElem.value, grade: gradeInputElem.value};
    this.genericService.AddGrade(stud, newgrade)
      .subscribe(response => {
          let r: any = response;
          this.errorMessage = r.result;
      })
  } 

}
